"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepare = void 0;
const clc = require("cli-color");
const ensureApiEnabled = require("../../ensureApiEnabled");
const functionsConfig = require("../../functionsConfig");
const getProjectId = require("../../getProjectId");
const utils_1 = require("../../utils");
const parseRuntimeAndValidateSDK_1 = require("../../parseRuntimeAndValidateSDK");
const functionsDeployHelper_1 = require("../../functionsDeployHelper");
const deploymentPlanner_1 = require("./deploymentPlanner");
const prompts_1 = require("./prompts");
const prepareFunctionsUpload_1 = require("../../prepareFunctionsUpload");
const gcp = require("../../gcp");
const validate = require("./validate");
const checkRuntimeDependencies_1 = require("./checkRuntimeDependencies");
const error_1 = require("../../error");
async function prepare(context, options, payload) {
    if (!options.config.has("functions")) {
        return;
    }
    const sourceDirName = options.config.get("functions.source");
    if (!sourceDirName) {
        throw new error_1.FirebaseError(`No functions code detected at default location (./functions), and no functions.source defined in firebase.json`);
    }
    const sourceDir = options.config.path(sourceDirName);
    const projectDir = options.config.projectDir;
    const projectId = getProjectId(options);
    const runtimeFromConfig = options.config.get("functions.runtime");
    context.runtimeChoice = parseRuntimeAndValidateSDK_1.getRuntimeChoice(sourceDir, runtimeFromConfig);
    const checkAPIsEnabled = await Promise.all([
        ensureApiEnabled.ensure(projectId, "cloudfunctions.googleapis.com", "functions"),
        ensureApiEnabled.check(projectId, "runtimeconfig.googleapis.com", "runtimeconfig", true),
        checkRuntimeDependencies_1.checkRuntimeDependencies(projectId, context.runtimeChoice),
    ]);
    context.runtimeConfigEnabled = checkAPIsEnabled[1];
    const firebaseConfig = await functionsConfig.getFirebaseConfig(options);
    context.firebaseConfig = firebaseConfig;
    utils_1.logBullet(clc.cyan.bold("functions:") +
        " preparing " +
        clc.bold(options.config.get("functions.source")) +
        " directory for uploading...");
    const source = await prepareFunctionsUpload_1.prepareFunctionsUpload(context, options);
    context.functionsSource = source;
    const defaultEnvVariables = {
        FIREBASE_CONFIG: JSON.stringify(context.firebaseConfig),
    };
    const functions = options.config.get("functions.triggers");
    functions.forEach((fn) => {
        fn.environmentVariables = defaultEnvVariables;
    });
    const includesScheduledFunctions = functions.some((fn) => fn.schedule);
    if (includesScheduledFunctions) {
        await Promise.all([
            ensureApiEnabled.ensure(projectId, "cloudscheduler.googleapis.com", "scheduler", false),
            ensureApiEnabled.ensure(projectId, "pubsub.googleapis.com", "pubsub", false),
        ]);
    }
    const byRegion = deploymentPlanner_1.functionsByRegion(projectId, functions);
    payload.functions = {
        byRegion,
        triggers: deploymentPlanner_1.allFunctions(byRegion),
    };
    validate.functionsDirectoryExists(options, sourceDirName);
    validate.packageJsonIsValid(sourceDirName, sourceDir, projectDir, !!runtimeFromConfig);
    context.filters = functionsDeployHelper_1.getFilterGroups(options);
    const localFnsInRelease = payload.functions.triggers.filter((fn) => {
        return functionsDeployHelper_1.functionMatchesAnyGroup(fn.name, context.filters);
    });
    const res = await gcp.cloudfunctions.listAllFunctions(context.projectId);
    if (res.unreachable) {
        const regionsInDeployment = Object.keys(payload.functions.byRegion);
        const unreachableRegionsInDeployment = regionsInDeployment.filter((region) => res.unreachable.includes(region));
        if (unreachableRegionsInDeployment) {
            throw new error_1.FirebaseError("The following Cloud Functions regions are currently unreachable:\n\t" +
                unreachableRegionsInDeployment.join("\n\t") +
                "\nThis deployment contains functions in those regions. Please try again in a few minutes, or exclude these regions from your deployment.");
        }
        else {
            utils_1.logLabeledWarning("functions", "The following Cloud Functions regions are currently unreachable:\n" +
                res.unreachable.join("\n") +
                "\nCloud Functions in these regions won't be deleted.");
        }
    }
    context.existingFunctions = res.functions;
    await prompts_1.promptForFailurePolicies(options, localFnsInRelease, context.existingFunctions);
}
exports.prepare = prepare;
